'use strict';

var fs = require('node:fs');
var path = require('node:path');
var stringHash = require('string-hash');
var findCacheDirectory = require('find-cache-dir');
var taddy = require('taddy');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var fs__namespace = /*#__PURE__*/_interopNamespaceDefault(fs);
var path__namespace = /*#__PURE__*/_interopNamespaceDefault(path);

const DEFAULT_CACHE_DIR = path__namespace.join(__dirname, '../cache');
function getCacheDir() {
    return (findCacheDirectory({ name: 'taddy', create: true }) || DEFAULT_CACHE_DIR);
}
const LAST_INDEX = 0;
const STYLES = [];
function getStylesState() {
    const { rules } = taddy.$css.ruleInjector.styleSheet;
    const newRules = rules.slice(LAST_INDEX);
    // there is also "appending" rules, that don't change the index, but change the rule
    // TODO: think how to handle that
    // LAST_INDEX = rules.length;
    const added = [...newRules].map((rule) => rule.cssText || '');
    STYLES.push(...added);
    return {
        added,
    };
}
function getMtime(filepath) {
    return Number(fs__namespace.statSync?.(filepath).mtime);
}
const filesMap = new Map();
const contentMap = new Map();
function readFileSync(filepath) {
    if (!fs__namespace.existsSync(filepath))
        return '';
    let data = '';
    try {
        const mtime = getMtime(filepath);
        if (filesMap.has(mtime)) {
            return filesMap.get(mtime);
        }
        data = fs__namespace.readFileSync(filepath).toString();
        if (!data)
            return '';
        filesMap.set(mtime, data);
        contentMap.set(data, mtime);
    }
    catch (e) {
        console.error('error', e);
    }
    return data;
}
function mkdir(filepath) {
    fs__namespace.mkdirSync(path__namespace.dirname(filepath), { recursive: true });
}
function appendFile(filepath, append) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs__namespace.appendFileSync(filepath, append);
}
function writeFile(filepath, code) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs__namespace.writeFileSync(filepath, code);
}
/** don't merge declarations in plugin, currently considering only dev mode */
taddy.$css.ruleInjector.styleSheet.options.mergeDeclarations = false;
class Output {
    env;
    config;
    filepath;
    defaultCacheDir = getCacheDir();
    constructor({ env, config }) {
        this.env = env;
        this.config = Object.assign({
            // getCSSFilename = (content) => `atoms-${stringHash(content)}.css`,
            cacheDir: this.defaultCacheDir,
        }, config);
        // const filename = this.resolveFilepath(this.config.cssFilename);
        // this.filepath = this.resolveFilepath(this.config.cssFilepath, filename);
        this.filepath = path__namespace.join(this.config.cacheDir, 'taddy.css');
        mkdir(this.filepath);
        // console.log('filepath', this.filepath);
        /* clean cache */
        // fs.writeFileSync(this.filepath, '');
    }
    resolveFilepath(getter, ...params) {
        if (typeof getter !== 'string') {
            // @ts-expect-error TODO: fix getter params
            return getter(...params);
        }
        return getter;
    }
    save({ sourceMap, filenameRelative, target, }) {
        const stylesData = readFileSync(this.filepath);
        const { added } = getStylesState();
        // console.log({added});
        // TODO: improve filter performance
        const diffStyles = added
            .filter((x) => !stylesData.includes(x))
            .join('');
        appendFile(this.filepath, diffStyles);
        // appendFile(this.filepath, sourceMap);
        const filenameHash = stringHash(`${filenameRelative}`).toString(36);
        const localFileCachePath = path__namespace.join(this.config.cacheDir, filenameHash + '.taddy.css');
        const localFileCacheData = readFileSync(localFileCachePath);
        const localFileCacheDiff = added
            .filter((x) => !localFileCacheData.includes(x))
            .join('');
        appendFile(localFileCachePath, localFileCacheDiff);
        const localFileContent = localFileCacheData + localFileCacheDiff;
        const localFileContentHash = stringHash(localFileContent).toString(36);
        const localFilename = path__namespace.join(filenameHash, localFileContentHash + '.taddy');
        const localFilepath = path__namespace.join(this.config.cacheDir, 
        // stringHash(`${filename}:${added}`) + '.taddy.css',
        localFilename);
        writeFile(localFilepath + '.css', localFileContent.replace(/}$/, sourceMap + '}'));
        const importBase = this.config.cacheDir === this.defaultCacheDir
            ? '.cache/taddy'
            : path__namespace.relative(path__namespace.dirname(filenameRelative), this.config.cacheDir);
        if (target === 'remix') {
            writeFile(localFilepath + '.cjs', `require('./${localFilename}.css');`);
            writeFile(localFilepath + '.js', `import './${localFilename}.css'`);
            return { importName: path__namespace.join(importBase, localFilename) };
        }
        return { importName: path__namespace.join(importBase, `${localFilename}.css`) };
    }
}

exports.Output = Output;
//# sourceMappingURL=Output.js.map
