'use strict';

var t = require('@babel/types');
var stringHash = require('string-hash');
var taddy = require('taddy');
var Processor = require('./Processor.js');
var BindingOptimizer = require('./helpers/BindingOptimizer.js');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

function isPathRemoved(path) {
    do {
        if (path?.removed)
            return true;
    } while ((path = path?.parentPath));
    return false;
}
const createHandlers = (config, options) => {
    const mixinsQueue = [];
    const proceedPaths = [];
    const cssQueue = [];
    const processor = new Processor.Processor({ config });
    const optimizer = new BindingOptimizer.BindingOptimizer();
    let counter = 0;
    const addHashId = (path) => {
        path.node.arguments.push(t__namespace.stringLiteral('__' +
            stringHash(options.filenameRelative + ':' + String(++counter)).toString(32)));
    };
    const handlers = {
        css: (path, { mixin = false } = {}) => {
            if (path.isMemberExpression() &&
                'name' in path.node.property &&
                path.node.property.name === 'mixin') {
                mixinsQueue.push(path.parentPath);
                return;
            }
            if (path.isCallExpression()) {
                cssQueue.push({ path, mixin });
                return;
            }
        },
        mixin: (path) => {
            mixinsQueue.push(path);
        },
    };
    function processCSS({ path, mixin, }) {
        const result = processor.run(path, {
            ...options,
            mixin,
        });
        if (!result)
            return;
        const { isStatic, optimizationPaths } = result;
        const cssLength = taddy.$css.ruleInjector.styleSheet.rules.join('').length;
        // console.log(options.state.file.opts.generatorOpts.sourceFileName);
        const source = options.state?.file.opts.generatorOpts?.sourceFileName;
        const locStart = path.node.loc?.start;
        if (source && locStart) {
            options.sourceMapGenerator?.addMapping({
                generated: {
                    line: 1,
                    column: cssLength,
                },
                source,
                original: locStart,
            });
        }
        proceedPaths.push({
            path,
            isStatic,
            optimizationPaths,
        });
        if (!mixin) {
            addHashId(path);
        }
    }
    return {
        handlers,
        finish() {
            cssQueue.forEach(processCSS);
            let isStatic = true;
            for (const x of proceedPaths) {
                isStatic = isStatic && x.isStatic;
                if (!config.optimizeBindings)
                    continue;
                for (const path of x.optimizationPaths) {
                    optimizer.process(path);
                }
            }
            for (const path of mixinsQueue) {
                if (config.optimizeBindings && isPathRemoved(path)) {
                    // the path binding should be already optimized
                    continue;
                }
                isStatic = false;
                processCSS({ path, mixin: true });
            }
            /**
             * If it's not a static file, map only static usages
             */
            if (!isStatic) {
                for (const x of proceedPaths) {
                    if (!x.isStatic)
                        continue;
                    const { path } = x;
                    path.replaceWith(t__namespace.callExpression(t__namespace.memberExpression(path.node.callee, t__namespace.identifier('static')), path.node.arguments));
                }
            }
            return { isStatic };
        },
    };
};

exports.createHandlers = createHandlers;
//# sourceMappingURL=handlers.js.map
