'use strict';

var evaluatePath = require('babel-helper-evaluate-path');
var config = require('../config.js');
var findBindings = require('./findBindings.js');
var buildCodeByPath = require('./buildCodeByPath.js');
require('./evaluate.worker.js');

// NOTE: bun doesn't support sync-rps
// TODO: support bun environment and avoid transpiling
const evaluateChunk = process.versions.bun
    ? undefined()
    : require('sync-rpc')(__dirname + '/evaluate.worker.js', 'Evaluate');
const macroRe = new RegExp(config.MACRO_NAME.replace('.', '\\.'), 'g');
function evaluate(currentPath, { exec = true }) {
    const result = evaluatePath(currentPath);
    if (result.confident) {
        findBindings.findBindings(currentPath);
        return { value: result.value };
    }
    if (!exec)
        return { error: new Error('EXEC_REQUIRED') };
    let content = '';
    try {
        const callbackName = '__taddy__';
        content = buildCodeByPath.buildCodeByPath(currentPath)
            // hack to avoid processing by babel-macro
            .replace(macroRe, config.PACKAGE_NAME)
            .concat(`\n\n;${callbackName}(${currentPath.toString()})`);
        // TODO: improve Hub type
        const { opts } = currentPath.hub.file;
        const { value, error } = evaluateChunk({
            content,
            filename: opts.filename,
            callbackName,
        });
        if (error)
            return { error };
        return { value: JSON.parse(value) };
    }
    catch (error) {
        // console.log('evaluate error', {content, error});
        return { error };
    }
}

exports.evaluate = evaluate;
//# sourceMappingURL=evaluate.js.map
