'use strict';

var t = require('@babel/types');
var core = require('@taddy/core');
var getObjectPropertyKey = require('./getObjectPropertyKey.js');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

function optimizeStaticStyles(path) {
    const props = [];
    let quasis = [];
    let expressions = [];
    let currQuasi = '';
    for (const propPath of path.get('properties')) {
        if (!propPath.isObjectProperty()) {
            throw new Error('Cant optimize this path');
        }
        const keyPath = propPath.get('key');
        const key = getObjectPropertyKey.getObjectPropertyKey(keyPath);
        if (!key)
            continue;
        const valuePath = propPath.get('value');
        if (key === 'className' || key === 'style' || key === core.VARS_KEY) {
            if (currQuasi) {
                quasis.push(t__namespace.templateElement({ raw: currQuasi }));
                props.push(t__namespace.objectProperty(t__namespace.templateLiteral(quasis, expressions), t__namespace.booleanLiteral(true)));
            }
            props.push(propPath.node);
            quasis = [];
            expressions = [];
            currQuasi = '';
            continue;
        }
        // append " " delimiter for the continuous value or if that's not the first quasi
        if (currQuasi || quasis.length !== 0) {
            currQuasi += ' ';
        }
        if (valuePath.isStringLiteral()) {
            currQuasi += key + valuePath.node.value;
        }
        else if (valuePath.isBooleanLiteral()) {
            currQuasi += key;
        }
        else {
            currQuasi += key;
            quasis.push(t__namespace.templateElement({ raw: currQuasi }));
            expressions.push(valuePath.node);
            currQuasi = '';
        }
    }
    if (currQuasi) {
        quasis.push(t__namespace.templateElement({ raw: currQuasi }));
    }
    if (expressions.length === quasis.length) {
        quasis.push(t__namespace.templateElement({ raw: '' }));
    }
    if (quasis.length) {
        if (props.length === 0) {
            path.replaceWith(t__namespace.templateLiteral(quasis, expressions));
            return;
        }
        props.push(t__namespace.objectProperty(t__namespace.templateLiteral(quasis, expressions), t__namespace.booleanLiteral(true)));
    }
    path.node.properties = props;
}

exports.optimizeStaticStyles = optimizeStaticStyles;
//# sourceMappingURL=optimizeStaticStyles.js.map
