'use strict';

var t = require('@babel/types');
var assert = require('node:assert');
var path = require('node:path');
var taddy = require('taddy');
var config = require('./config.js');
require('./helpers/utils.js');
var taggedTemplateToObject = require('./helpers/taggedTemplateToObject.js');
var handlers = require('./handlers.js');
var Output = require('./Output.js');
var sourceMaps = require('./source-maps.js');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

function mapCompileOptions({ filename, evaluate = true, unstable_typescript = false, unstable_CSSVariableFallback = true, unstable_optimizeBindings = true, unstable_useTaggedTemplateLiterals = false, unstable_target = 'auto', }) {
    const getTarget = () => {
        if (unstable_target !== 'auto')
            return unstable_target;
        const extname = path.extname(filename);
        if (extname === '.vue')
            return 'vue';
        return 'auto';
    };
    return {
        evaluate,
        typescript: unstable_typescript,
        CSSVariableFallback: unstable_CSSVariableFallback,
        optimizeBindings: unstable_optimizeBindings,
        useTaggedTemplateLiterals: unstable_useTaggedTemplateLiterals,
        target: getTarget(),
    };
}
const findImportPath = (program) => {
    let importPath = undefined;
    program.traverse({
        ImportDeclaration(p) {
            const { node } = p;
            if (!(node.source.value === config.PACKAGE_NAME ||
                node.source.value === config.MACRO_NAME)) {
                return;
            }
            importPath = p;
            importPath.node.source = t__namespace.stringLiteral('taddy');
            p.stop();
        },
    });
    return importPath;
};
let output;
let sourceMapGenerator;
function macro({ references, babel, state, config: _config = {}, }) {
    const { env = config.getEnv(babel), ...config$1 } = _config;
    if (undefined(state)) ;
    const program = state.file.path;
    const code = state.file.code;
    const { filename, sourceRoot, root, filenameRelative = path.relative(sourceRoot || root || process.cwd(), filename), } = state.file.opts;
    assert(filename, 'Filename is required');
    assert(filenameRelative, 'FilenameRelative is required');
    const importPath = findImportPath(program);
    if (!importPath)
        return;
    sourceMapGenerator = sourceMaps.makeSourceMapGenerator(state.file);
    sourceMapGenerator.setSourceContent(filename, code);
    // TODO: implement "createCSS" and pass it
    taddy.$css.ruleInjector.reset();
    const importCache = new Map();
    const { useTaggedTemplateLiterals, ...compileOptions } = mapCompileOptions({
        filename,
        ...config$1.compileOptions,
    });
    switch (compileOptions.target) {
        case 'vue': {
            require('taddy/vue');
            break;
        }
    }
    const { handlers: handlers$1, finish } = handlers.createHandlers(compileOptions, {
        state,
        sourceMapGenerator,
        env,
        filename,
        filenameRelative,
        code,
        addImport(name) {
            if (!importCache.has(name)) {
                const specifier = t__namespace.importSpecifier(t__namespace.identifier(`_${name}_`), t__namespace.identifier(name));
                importCache.set(name, specifier);
                importPath.node.specifiers.push(specifier);
            }
            return importCache.get(name).local;
        },
    });
    for (const key in references) {
        if (!(key in handlers$1))
            continue;
        for (const ref of references[key]) {
            let currentPath = ref.parentPath;
            let currentKey = key;
            if (key === 'css' &&
                currentPath?.isMemberExpression() &&
                'name' in currentPath.node.property &&
                currentPath.node.property.name === 'mixin') {
                currentPath = currentPath.parentPath;
                currentKey = 'mixin';
            }
            if (useTaggedTemplateLiterals &&
                (currentKey === 'css' || currentKey === 'mixin') &&
                currentPath?.isTaggedTemplateExpression()) {
                const obj = taggedTemplateToObject.taggedTemplateToObject(currentPath);
                currentPath.replaceWith(t__namespace.callExpression(currentPath.node.tag, [obj]));
            }
            handlers$1[currentKey](currentPath);
        }
    }
    const { isStatic } = finish();
    if (isStatic) {
        importPath.node.source = t__namespace.stringLiteral('@taddy/core');
    }
    output = output || new Output.Output({ env, config: config$1.outputOptions });
    const sourceMap = sourceMaps.convertGeneratorToComment(sourceMapGenerator);
    const result = output.save({
        sourceMap,
        filenameRelative,
        target: compileOptions.target,
    });
    importPath.insertAfter(t__namespace.importDeclaration([], t__namespace.stringLiteral(result.importName)));
    return {
        keepImports: true,
    };
}

exports.macro = macro;
//# sourceMappingURL=macro-plugin.js.map
